@@ -33,28 +33,31 @@ AUTHORS:
 # ****************************************************************************
 
 import importlib
-import random
+import json
 import os
+import random
+import shlex
 import sys
 import time
-import json
-import shlex
 import types
-import sage.misc.flatten
-import sage.misc.randstate as randstate
-from sage.structure.sage_object import SageObject
-from sage.env import DOT_SAGE, SAGE_LIB, SAGE_SRC, SAGE_VENV, SAGE_EXTCODE
-from sage.misc.temporary_file import tmp_dir
+
 from cysignals.signals import AlarmInterrupt, init_cysignals
 
-from .sources import FileDocTestSource, DictAsObject, get_basename
-from .forker import DocTestDispatcher
-from .reporting import DocTestReporter
-from .util import Timer, count_noun, dict_difference
-from .external import available_software
-from .parsing import parse_optional_tags, parse_file_optional_tags, unparse_optional_tags, \
-     nodoctest_regex, optionaltag_regex, optionalfiledirective_regex
-
+import sage.misc.flatten
+from sage.doctest.external import available_software
+from sage.doctest.forker import DocTestDispatcher
+from sage.doctest.parsing import (
+    optional_tag_regex,
+    parse_file_optional_tags,
+    unparse_optional_tags,
+)
+from sage.doctest.reporting import DocTestReporter
+from sage.doctest.sources import DictAsObject, FileDocTestSource, get_basename
+from sage.doctest.util import Timer, count_noun, dict_difference
+from sage.env import DOT_SAGE, SAGE_EXTCODE, SAGE_LIB, SAGE_SRC
+from sage.misc import randstate
+from sage.misc.temporary_file import tmp_dir
+from sage.structure.sage_object import SageObject
 
 # Optional tags which are always automatically added
 
@@ -317,7 +320,7 @@ def skipfile(filename, tested_optional_tags=False, *,
                 log(f"Skipping '{filename}' because module {e.name} cannot be imported")
             return True
 
-    with open(filename) as F:
+    with open(filename, encoding="utf-8") as F:
         file_optional_tags = parse_file_optional_tags(enumerate(F))
 
     if 'not tested' in file_optional_tags:
@@ -453,6 +456,10 @@ class DocTestController(SageObject):
                 print("Debugging is not compatible with logging, disabling logfile.")
             options.serial = True
             options.logfile = None
+        try:
+            from signal import SIGCHLD
+        except ImportError:
+            options.serial = True
         if options.serial:
             options.nthreads = 1
         if options.verbose:
@@ -466,7 +473,7 @@ class DocTestController(SageObject):
                 s = options.hide.lower()
                 options.hide = set(s.split(','))
                 for h in options.hide:
-                    if not optionaltag_regex.search(h):
+                    if not optional_tag_regex.search(h):
                         raise ValueError('invalid optional tag {!r}'.format(h))
             if 'all' in options.hide:
                 options.hide.discard('all')
@@ -498,7 +505,7 @@ class DocTestController(SageObject):
                         if pkg.name in options.hide:
                             continue
                         # Skip features for which we have a more specific runtime feature test.
-                        if pkg.name in ['4ti2', 'bliss', 'buckygen', 'coxeter3', 'cvxpy', 'database_graphs', 'eclib', 'ecm', 'fricas', 'frobby', 'gfan', 'giac', 'jmol', 'kenzo', 'latte_int', 'macaulay2', 'mcqd', 'meataxe', 'msolve', 'palp', 'plantri', 'polytopes_db', 'qepcad', 'rubiks', 'sirocco', 'sympow', 'tdlib', 'topcom']:
+                        if pkg.name in ['4ti2', 'bliss', 'buckygen', 'coxeter3', 'cvxpy', 'cunningham_tables', 'database_graphs', 'database_jones_numfield', 'database_mutation_class', 'database_symbolic_data', 'eclib', 'ecm', 'fricas', 'frobby', 'gfan', 'giac', 'jmol', 'kenzo', 'latte_int', 'macaulay2', 'mcqd', 'meataxe', 'msolve', 'palp', 'plantri', 'polytopes_db', 'qepcad', 'rubiks', 'sirocco', 'sympow', 'tachyon', 'tdlib', 'topcom']:
                             continue
                         if pkg.is_installed() and pkg.installed_version == pkg.remote_version:
                             options.optional.add(pkg.name)
@@ -509,10 +516,10 @@ class DocTestController(SageObject):
                 # Check that all tags are valid
                 for o in options.optional:
                     if o.startswith('!'):
-                        if not optionaltag_regex.search(o[1:]):
+                        if not optional_tag_regex.search(o[1:]):
                             raise ValueError('invalid optional tag {!r}'.format(o))
                         options.disabled_optional.add(o[1:])
-                    elif not optionaltag_regex.search(o):
+                    elif not optional_tag_regex.search(o):
                         raise ValueError('invalid optional tag {!r}'.format(o))
 
                 options.optional |= auto_optional_tags
@@ -532,7 +539,7 @@ class DocTestController(SageObject):
                 else:
                     # Check that all tags are valid
                     for o in options.probe:
-                        if not optionaltag_regex.search(o):
+                        if not optional_tag_regex.search(o):
                             raise ValueError('invalid optional tag {!r}'.format(o))
 
         self.options = options
@@ -574,6 +581,10 @@ class DocTestController(SageObject):
         else:
             self.logger = Logger(self._real_stdout, self.logfile)
 
+        # Do not assume that sage.misc.misc has been loaded,
+        # which creates DOT_SAGE as a side effect.
+        os.makedirs(DOT_SAGE, mode=0o700, exist_ok=True)
+
         self.stats = {}
         self.load_stats(options.stats_path)
         self.baseline_stats = {}
@@ -891,7 +902,7 @@ class DocTestController(SageObject):
             Doctesting ...
         """
         opj = os.path.join
-        from sage.env import SAGE_SRC, SAGE_DOC_SRC, SAGE_ROOT, SAGE_ROOT_GIT, SAGE_DOC
+        from sage.env import SAGE_DOC, SAGE_DOC_SRC, SAGE_ROOT, SAGE_ROOT_GIT, SAGE_SRC
         # SAGE_ROOT_GIT can be None on distributions which typically
         # only have the SAGE_LOCAL install tree but not SAGE_ROOT
         if SAGE_ROOT_GIT is not None:
@@ -1040,10 +1051,9 @@ class DocTestController(SageObject):
                                             bool(self.options.optional),
                                             if_installed=self.options.if_installed):
                                 yield os.path.join(root, file)
-                else:
-                    if not skipfile(path, bool(self.options.optional),
-                                    if_installed=self.options.if_installed, log=self.log):  # log when directly specified filenames are skipped
-                        yield path
+                elif not skipfile(path, bool(self.options.optional),
+                                if_installed=self.options.if_installed, log=self.log):  # log when directly specified filenames are skipped
+                    yield path
         self.sources = [FileDocTestSource(path, self.options) for path in expand()]
 
     def filter_sources(self):
@@ -1352,12 +1362,12 @@ class DocTestController(SageObject):
         opt = self.options
 
         if opt.gdb:
-            cmd = f'''exec gdb --eval-command="run" --args '''
+            cmd = '''exec gdb --eval-command="run" --args '''
             flags = ""
             if opt.logfile:
                 sage_cmd += f" --logfile {shlex.quote(opt.logfile)}"
         elif opt.lldb:
-            cmd = f'''exec lldb --one-line "process launch" --one-line "cont" -- '''
+            cmd = '''exec lldb --one-line "process launch" --one-line "cont" -- '''
             flags = ""
         else:
             if opt.logfile is None:
@@ -1549,7 +1559,7 @@ class DocTestController(SageObject):
             return self.run_val_gdb()
         else:
             self.create_run_id()
-            from sage.env import SAGE_ROOT_GIT, SAGE_LOCAL, SAGE_VENV
+            from sage.env import SAGE_LOCAL, SAGE_ROOT_GIT, SAGE_VENV
             # SAGE_ROOT_GIT can be None on distributions which typically
             # only have the SAGE_LOCAL install tree but not SAGE_ROOT
             if (SAGE_ROOT_GIT is not None) and os.path.isdir(SAGE_ROOT_GIT):
@@ -1684,9 +1694,9 @@ def run_doctests(module, options=None):
         IP = get_ipython()
         if IP is not None:
             old_color = IP.colors
-            IP.run_line_magic('colors', 'NoColor')
+            IP.run_line_magic('colors', 'nocolor')
             old_config_color = IP.config.TerminalInteractiveShell.colors
-            IP.config.TerminalInteractiveShell.colors = 'NoColor'
+            IP.config.TerminalInteractiveShell.colors = 'nocolor'
 
     try:
         DC.run()
